@@ -1,5 +1,3 @@
-from __future__ import print_function, absolute_import
-
 import ctypes
 from ctypes import CFUNCTYPE, c_int
 from ctypes.util import find_library
@@ -12,11 +10,12 @@ import subprocess
 import sys
 import unittest
 from contextlib import contextmanager
+from tempfile import mkstemp
 
-from llvmlite import six, ir
+from llvmlite import ir
 from llvmlite import binding as llvm
 from llvmlite.binding import ffi
-from . import TestCase
+from llvmlite.tests import TestCase
 
 
 # arvm7l needs extra ABI symbols to link successfully
@@ -39,11 +38,12 @@ def no_de_locale():
 asm_sum = r"""
     ; ModuleID = '<string>'
     target triple = "{triple}"
+    %struct.glob_type = type {{ i64, [2 x i64]}}
 
     @glob = global i32 0
     @glob_b = global i8 0
     @glob_f = global float 1.5
-    @glob_struct = global {{ i64, [2 x i64]}} {{i64 0, [2 x i64] [i64 0, i64 0]}}
+    @glob_struct = global %struct.glob_type {{i64 0, [2 x i64] [i64 0, i64 0]}}
 
     define i32 @sum(i32 %.1, i32 %.2) {{
       %.3 = add i32 %.1, %.2
@@ -65,6 +65,7 @@ asm_sum2 = r"""
 asm_mul = r"""
     ; ModuleID = '<string>'
     target triple = "{triple}"
+    @mul_glob = global i32 0
 
     define i32 @mul(i32 %.1, i32 %.2) {{
       %.3 = mul i32 %.1, %.2
@@ -123,6 +124,54 @@ asm_inlineasm = r"""
     }}
     """
 
+asm_global_ctors = r"""
+    ; ModuleID = "<string>"
+    target triple = "{triple}"
+
+    @A = global i32 undef
+
+    define void @ctor_A()
+    {{
+      store i32 10, i32* @A
+      ret void
+    }}
+
+    define void @dtor_A()
+    {{
+      store i32 20, i32* @A
+      ret void
+    }}
+
+    define i32 @foo()
+    {{
+      %.2 = load i32, i32* @A
+      %.3 = add i32 %.2, 2
+      ret i32 %.3
+    }}
+
+    @llvm.global_ctors = appending global [1 x {{i32, void ()*, i8*}}] [{{i32, void ()*, i8*}} {{i32 0, void ()* @ctor_A, i8* null}}]
+    @llvm.global_dtors = appending global [1 x {{i32, void ()*, i8*}}] [{{i32, void ()*, i8*}} {{i32 0, void ()* @dtor_A, i8* null}}]
+    """  # noqa E501
+
+
+asm_nonalphanum_blocklabel = """; ModuleID = ""
+target triple = "unknown-unknown-unknown"
+target datalayout = ""
+
+define i32 @"foo"() 
+{
+"<>!*''#":
+  ret i32 12345
+}
+"""  # noqa W291 # trailing space needed for match later
+
+
+asm_attributes = r"""
+declare void @a_readonly_func(i8 *) readonly
+
+declare i8* @a_arg0_return_func(i8* returned, i32*)
+"""
+
 
 class BaseTest(TestCase):
 
@@ -143,9 +192,9 @@ class BaseTest(TestCase):
         # This will probably put any existing garbage in gc.garbage again
         del self.old_garbage
 
-    def module(self, asm=asm_sum):
+    def module(self, asm=asm_sum, context=None):
         asm = asm.format(triple=llvm.get_default_triple())
-        mod = llvm.parse_assembly(asm)
+        mod = llvm.parse_assembly(asm, context)
         return mod
 
     def glob(self, name='glob', mod=None):
@@ -153,9 +202,9 @@ class BaseTest(TestCase):
             mod = self.module()
         return mod.get_global_variable(name)
 
-    def target_machine(self):
+    def target_machine(self, *, jit):
         target = llvm.Target.from_default_triple()
-        return target.create_target_machine()
+        return target.create_target_machine(jit=jit)
 
 
 class TestDependencies(BaseTest):
@@ -163,8 +212,10 @@ class TestDependencies(BaseTest):
     Test DLL dependencies are within a certain expected set.
     """
 
-    @unittest.skipUnless(sys.platform.startswith('linux'), "Linux-specific test")
-    @unittest.skipUnless(os.environ.get('LLVMLITE_DIST_TEST'), "Distribution-specific test")
+    @unittest.skipUnless(sys.platform.startswith('linux'),
+                         "Linux-specific test")
+    @unittest.skipUnless(os.environ.get('LLVMLITE_DIST_TEST'),
+                         "Distribution-specific test")
     def test_linux(self):
         lib_path = ffi.lib._name
         env = os.environ.copy()
@@ -213,6 +264,20 @@ class TestMisc(BaseTest):
         self.assertIn("parsing error", s)
         self.assertIn("invalid operand type", s)
 
+    def test_nonalphanum_block_name(self):
+        mod = ir.Module()
+        ft = ir.FunctionType(ir.IntType(32), [])
+        fn = ir.Function(mod, ft, "foo")
+        bd = ir.IRBuilder(fn.append_basic_block(name="<>!*''#"))
+        bd.ret(ir.Constant(ir.IntType(32), 12345))
+        asm = str(mod)
+        self.assertEqual(asm, asm_nonalphanum_blocklabel)
+
+    def test_global_context(self):
+        gcontext1 = llvm.context.get_global_context()
+        gcontext2 = llvm.context.get_global_context()
+        assert gcontext1 == gcontext2
+
     def test_dylib_symbols(self):
         llvm.add_symbol("__xyzzy", 1234)
         llvm.add_symbol("__xyzzy", 5678)
@@ -238,23 +303,13 @@ class TestMisc(BaseTest):
         self.assertEqual(default_parts[0], triple_parts[0])
 
     def test_get_host_cpu_features(self):
-        try:
-            features = llvm.get_host_cpu_features()
-        except RuntimeError:
-            # Allow non-x86 arch to pass even if an RuntimeError is raised
-            triple = llvm.get_process_triple()
-            # For now, we know for sure that x86 is supported.
-            # We can restrict the test if we know this works on other arch.
-            is_x86 = triple.startswith('x86')
-            self.assertFalse(is_x86,
-                             msg="get_host_cpu_features() should not raise")
-            return
+        features = llvm.get_host_cpu_features()
         # Check the content of `features`
         self.assertIsInstance(features, dict)
         self.assertIsInstance(features, llvm.FeatureMap)
         for k, v in features.items():
             self.assertIsInstance(k, str)
-            self.assertTrue(k)  # feature string cannot be empty
+            self.assertTrue(k)  # single feature string cannot be empty
             self.assertIsInstance(v, bool)
         self.assertIsInstance(features.flatten(), str)
 
@@ -265,7 +320,10 @@ class TestMisc(BaseTest):
         self.assertIsNotNone(re.match(regex, "+aa"))
         self.assertIsNotNone(re.match(regex, "+a,-bb"))
         # check CpuFeature.flatten()
-        self.assertIsNotNone(re.match(regex, features.flatten()))
+        if len(features) == 0:
+            self.assertEqual(features.flatten(), "")
+        else:
+            self.assertIsNotNone(re.match(regex, features.flatten()))
 
     def test_get_host_cpu_name(self):
         cpu = llvm.get_host_cpu_name()
@@ -297,7 +355,9 @@ class TestMisc(BaseTest):
 
     def test_version(self):
         major, minor, patch = llvm.llvm_version_info
-        self.assertEqual((major, minor), (4, 0))
+        # one of these can be valid
+        valid = [(9, 0), (8, 0), (7, 0), (7, 1)]
+        self.assertIn((major, minor), valid)
         self.assertIn(patch, range(10))
 
     def test_check_jit_execution(self):
@@ -387,6 +447,18 @@ class TestModuleRef(BaseTest):
         del mod
         str(fn.module)
 
+    def test_get_struct_type(self):
+        mod = self.module()
+        st_ty = mod.get_struct_type("struct.glob_type")
+        self.assertEqual(st_ty.name, "struct.glob_type")
+        # also match struct names of form "%struct.glob_type.{some_index}"
+        self.assertIsNotNone(re.match(
+            r'%struct\.glob_type(\.[\d]+)? = type { i64, \[2 x i64\] }',
+            str(st_ty)))
+
+        with self.assertRaises(NameError):
+            mod.get_struct_type("struct.doesnt_exist")
+
     def test_get_global_variable(self):
         mod = self.module()
         gv = mod.get_global_variable("glob")
@@ -417,11 +489,24 @@ class TestModuleRef(BaseTest):
         self.assertEqual(len(funcs), 1)
         self.assertEqual(funcs[0].name, "sum")
 
+    def test_structs(self):
+        mod = self.module()
+        it = mod.struct_types
+        del mod
+        structs = list(it)
+        self.assertEqual(len(structs), 1)
+        self.assertIsNotNone(re.match(r'struct\.glob_type(\.[\d]+)?',
+                                      structs[0].name))
+        self.assertIsNotNone(re.match(
+            r'%struct\.glob_type(\.[\d]+)? = type { i64, \[2 x i64\] }',
+            str(structs[0])))
+
     def test_link_in(self):
         dest = self.module()
         src = self.module(asm_mul)
         dest.link_in(src)
-        self.assertEqual(sorted(f.name for f in dest.functions), ["mul", "sum"])
+        self.assertEqual(
+            sorted(f.name for f in dest.functions), ["mul", "sum"])
         dest.get_function("mul")
         dest.close()
         with self.assertRaises(ctypes.ArgumentError):
@@ -431,7 +516,8 @@ class TestModuleRef(BaseTest):
         dest = self.module()
         src2 = self.module(asm_mul)
         dest.link_in(src2, preserve=True)
-        self.assertEqual(sorted(f.name for f in dest.functions), ["mul", "sum"])
+        self.assertEqual(
+            sorted(f.name for f in dest.functions), ["mul", "sum"])
         dest.close()
         self.assertEqual(sorted(f.name for f in src2.functions), ["mul"])
         src2.get_function("mul")
@@ -448,8 +534,8 @@ class TestModuleRef(BaseTest):
     def test_as_bitcode(self):
         mod = self.module()
         bc = mod.as_bitcode()
-        # Refer to http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00064
-        # and http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00092
+        # Refer to http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00064  # noqa E501
+        # and http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00092  # noqa E501
         bitcode_wrapper_magic = b'\xde\xc0\x17\x0b'
         bitcode_magic = b'BC'
         self.assertTrue(bc.startswith(bitcode_magic) or
@@ -459,12 +545,22 @@ class TestModuleRef(BaseTest):
         with self.assertRaises(RuntimeError) as cm:
             llvm.parse_bitcode(b"")
         self.assertIn("LLVM bitcode parsing error", str(cm.exception))
-        self.assertIn("Invalid bitcode signature", str(cm.exception))
+        # for llvm < 9
+        if llvm.llvm_version_info[0] < 9:
+            self.assertIn("Invalid bitcode signature", str(cm.exception))
+        else:
+            self.assertIn(
+                "file too small to contain bitcode header", str(cm.exception),
+            )
 
     def test_bitcode_roundtrip(self):
-        bc = self.module().as_bitcode()
-        mod = llvm.parse_bitcode(bc)
+        # create a new context to avoid struct renaming
+        context1 = llvm.create_context()
+        bc = self.module(context=context1).as_bitcode()
+        context2 = llvm.create_context()
+        mod = llvm.parse_bitcode(bc, context2)
         self.assertEqual(mod.as_bitcode(), bc)
+
         mod.get_function("sum")
         mod.get_global_variable("glob")
 
@@ -664,9 +760,9 @@ class JITWithTMTestMixin(JITTestMixin):
 
     def test_emit_assembly(self):
         """Test TargetMachineRef.emit_assembly()"""
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=True)
         mod = self.module()
-        ee = self.jit(mod, target_machine)
+        ee = self.jit(mod, target_machine)  # noqa F841 # Keeps pointers alive
         raw_asm = target_machine.emit_assembly(mod)
         self.assertIn("sum", raw_asm)
         target_machine.set_asm_verbosity(True)
@@ -676,11 +772,11 @@ class JITWithTMTestMixin(JITTestMixin):
 
     def test_emit_object(self):
         """Test TargetMachineRef.emit_object()"""
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=True)
         mod = self.module()
-        ee = self.jit(mod, target_machine)
+        ee = self.jit(mod, target_machine)  # noqa F841 # Keeps pointers alive
         code_object = target_machine.emit_object(mod)
-        self.assertIsInstance(code_object, six.binary_type)
+        self.assertIsInstance(code_object, bytes)
         if sys.platform.startswith('linux'):
             # Sanity check
             self.assertIn(b"ELF", code_object[:10])
@@ -693,7 +789,7 @@ class TestMCJit(BaseTest, JITWithTMTestMixin):
 
     def jit(self, mod, target_machine=None):
         if target_machine is None:
-            target_machine = self.target_machine()
+            target_machine = self.target_machine(jit=True)
         return llvm.create_mcjit_compiler(mod, target_machine)
 
 
@@ -750,7 +846,10 @@ class TestValueRef(BaseTest):
     def test_add_function_attribute(self):
         mod = self.module()
         fn = mod.get_function("sum")
-        fn.add_function_attribute("zext")
+        fn.add_function_attribute("nocapture")
+        with self.assertRaises(ValueError) as raises:
+            fn.add_function_attribute("zext")
+        self.assertEqual(str(raises.exception), "no such attribute 'zext'")
 
     def test_module(self):
         mod = self.module()
@@ -761,7 +860,37 @@ class TestValueRef(BaseTest):
         mod = self.module()
         glob = mod.get_global_variable("glob")
         tp = glob.type
-        self.assertIsInstance(tp, ffi.LLVMTypeRef)
+        self.assertIsInstance(tp, llvm.TypeRef)
+
+    def test_type_name(self):
+        mod = self.module()
+        glob = mod.get_global_variable("glob")
+        tp = glob.type
+        self.assertEqual(tp.name, "")
+        st = mod.get_global_variable("glob_struct")
+        self.assertIsNotNone(re.match(r"struct\.glob_type(\.[\d]+)?",
+                                      st.type.element_type.name))
+
+    def test_type_printing_variable(self):
+        mod = self.module()
+        glob = mod.get_global_variable("glob")
+        tp = glob.type
+        self.assertEqual(str(tp), 'i32*')
+
+    def test_type_printing_function(self):
+        mod = self.module()
+        fn = mod.get_function("sum")
+        self.assertEqual(str(fn.type), "i32 (i32, i32)*")
+
+    def test_type_printing_struct(self):
+        mod = self.module()
+        st = mod.get_global_variable("glob_struct")
+        self.assertTrue(st.type.is_pointer)
+        self.assertIsNotNone(re.match(r'%struct\.glob_type(\.[\d]+)?\*',
+                                      str(st.type)))
+        self.assertIsNotNone(re.match(
+            r"%struct\.glob_type(\.[\d]+)? = type { i64, \[2 x i64\] }",
+            str(st.type.element_type)))
 
     def test_close(self):
         glob = self.glob()
@@ -774,6 +903,89 @@ class TestValueRef(BaseTest):
         self.assertFalse(defined.is_declaration)
         self.assertTrue(declared.is_declaration)
 
+    def test_module_global_variables(self):
+        mod = self.module(asm_sum)
+        gvars = list(mod.global_variables)
+        self.assertEqual(len(gvars), 4)
+        for v in gvars:
+            self.assertTrue(v.is_global)
+
+    def test_module_functions(self):
+        mod = self.module()
+        funcs = list(mod.functions)
+        self.assertEqual(len(funcs), 1)
+        func = funcs[0]
+        self.assertTrue(func.is_function)
+        self.assertEqual(func.name, 'sum')
+
+        with self.assertRaises(ValueError):
+            func.instructions
+        with self.assertRaises(ValueError):
+            func.operands
+        with self.assertRaises(ValueError):
+            func.opcode
+
+    def test_function_arguments(self):
+        mod = self.module()
+        func = mod.get_function('sum')
+        self.assertTrue(func.is_function)
+        args = list(func.arguments)
+        self.assertEqual(len(args), 2)
+        self.assertTrue(args[0].is_argument)
+        self.assertTrue(args[1].is_argument)
+        self.assertEqual(args[0].name, '.1')
+        self.assertEqual(str(args[0].type), 'i32')
+        self.assertEqual(args[1].name, '.2')
+        self.assertEqual(str(args[1].type), 'i32')
+
+        with self.assertRaises(ValueError):
+            args[0].blocks
+        with self.assertRaises(ValueError):
+            args[0].arguments
+
+    def test_function_blocks(self):
+        func = self.module().get_function('sum')
+        blocks = list(func.blocks)
+        self.assertEqual(len(blocks), 1)
+        block = blocks[0]
+        self.assertTrue(block.is_block)
+
+    def test_block_instructions(self):
+        func = self.module().get_function('sum')
+        insts = list(list(func.blocks)[0].instructions)
+        self.assertEqual(len(insts), 3)
+        self.assertTrue(insts[0].is_instruction)
+        self.assertTrue(insts[1].is_instruction)
+        self.assertTrue(insts[2].is_instruction)
+        self.assertEqual(insts[0].opcode, 'add')
+        self.assertEqual(insts[1].opcode, 'add')
+        self.assertEqual(insts[2].opcode, 'ret')
+
+    def test_instruction_operands(self):
+        func = self.module().get_function('sum')
+        add = list(list(func.blocks)[0].instructions)[0]
+        self.assertEqual(add.opcode, 'add')
+        operands = list(add.operands)
+        self.assertEqual(len(operands), 2)
+        self.assertTrue(operands[0].is_operand)
+        self.assertTrue(operands[1].is_operand)
+        self.assertEqual(operands[0].name, '.1')
+        self.assertEqual(str(operands[0].type), 'i32')
+        self.assertEqual(operands[1].name, '.2')
+        self.assertEqual(str(operands[1].type), 'i32')
+
+    def test_function_attributes(self):
+        mod = self.module(asm_attributes)
+        for func in mod.functions:
+            attrs = list(func.attributes)
+            if func.name == 'a_readonly_func':
+                self.assertEqual(attrs, [b'readonly'])
+            elif func.name == 'a_arg0_return_func':
+                self.assertEqual(attrs, [])
+                args = list(func.arguments)
+                self.assertEqual(list(args[0].attributes), [b'returned'])
+                self.assertEqual(list(args[1].attributes), [])
+
 
 class TestTarget(BaseTest):
 
@@ -781,7 +993,7 @@ class TestTarget(BaseTest):
         f = llvm.Target.from_triple
         with self.assertRaises(RuntimeError) as cm:
             f("foobar")
-        self.assertIn("No available targets are compatible with this triple",
+        self.assertIn("No available targets are compatible with",
                       str(cm.exception))
         triple = llvm.get_default_triple()
         target = f(triple)
@@ -825,16 +1037,36 @@ class TestTargetData(BaseTest):
         glob = self.glob()
         self.assertEqual(td.get_abi_size(glob.type), 8)
 
+    def test_get_pointee_abi_size(self):
+        td = self.target_data()
+
+        glob = self.glob()
+        self.assertEqual(td.get_pointee_abi_size(glob.type), 4)
+
+        glob = self.glob("glob_struct")
+        self.assertEqual(td.get_pointee_abi_size(glob.type), 24)
+
+    def test_get_struct_element_offset(self):
+        td = self.target_data()
+        glob = self.glob("glob_struct")
+
+        with self.assertRaises(ValueError):
+            td.get_element_offset(glob.type, 0)
+
+        struct_type = glob.type.element_type
+        self.assertEqual(td.get_element_offset(struct_type, 0), 0)
+        self.assertEqual(td.get_element_offset(struct_type, 1), 8)
+
 
 class TestTargetMachine(BaseTest):
 
     def test_add_analysis_passes(self):
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         pm = llvm.create_module_pass_manager()
         tm.add_analysis_passes(pm)
 
     def test_target_data_from_tm(self):
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         td = tm.target_data
         mod = self.module()
         gv_i32 = mod.get_global_variable("glob")
@@ -861,14 +1093,14 @@ class TestPassManagerBuilder(BaseTest):
 
     def test_opt_level(self):
         pmb = self.pmb()
-        self.assertIsInstance(pmb.opt_level, six.integer_types)
+        self.assertIsInstance(pmb.opt_level, int)
         for i in range(4):
             pmb.opt_level = i
             self.assertEqual(pmb.opt_level, i)
 
     def test_size_level(self):
         pmb = self.pmb()
-        self.assertIsInstance(pmb.size_level, six.integer_types)
+        self.assertIsInstance(pmb.size_level, int)
         for i in range(4):
             pmb.size_level = i
             self.assertEqual(pmb.size_level, i)
@@ -880,13 +1112,6 @@ class TestPassManagerBuilder(BaseTest):
         for i in (25, 80, 350):
             pmb.inlining_threshold = i
 
-    def test_disable_unit_at_a_time(self):
-        pmb = self.pmb()
-        self.assertIsInstance(pmb.disable_unit_at_a_time, bool)
-        for b in (True, False):
-            pmb.disable_unit_at_a_time = b
-            self.assertEqual(pmb.disable_unit_at_a_time, b)
-
     def test_disable_unroll_loops(self):
         pmb = self.pmb()
         self.assertIsInstance(pmb.disable_unroll_loops, bool)
@@ -949,9 +1174,28 @@ class TestModulePassManager(BaseTest, PassManagerTestMixin):
         orig_asm = str(mod)
         pm.run(mod)
         opt_asm = str(mod)
-        # Quick check that optimizations were run
-        self.assertIn("%.3", orig_asm)
-        self.assertNotIn("%.3", opt_asm)
+        # Quick check that optimizations were run, should get:
+        # define i32 @sum(i32 %.1, i32 %.2) local_unnamed_addr #0 {
+        # %.X = add i32 %.2, %.1
+        # ret i32 %.X
+        # }
+        # where X in %.X is 3 or 4
+        opt_asm_split = opt_asm.splitlines()
+        for idx, l in enumerate(opt_asm_split):
+            if l.strip().startswith('ret i32'):
+                toks = {'%.3', '%.4'}
+                for t in toks:
+                    if t in l:
+                        break
+                else:
+                    raise RuntimeError("expected tokens not found")
+                othertoken = (toks ^ {t}).pop()
+
+                self.assertIn("%.3", orig_asm)
+                self.assertNotIn(othertoken, opt_asm)
+                break
+        else:
+            raise RuntimeError("expected IR not found")
 
 
 class TestFunctionPassManager(BaseTest, PassManagerTestMixin):
@@ -1089,6 +1333,32 @@ class TestTypeParsing(BaseTest):
             gv.initializer = ir.Constant(typ, [1])
 
 
+class TestGlobalConstructors(TestMCJit):
+    def test_global_ctors_dtors(self):
+        # test issue #303
+        # (https://github.com/numba/llvmlite/issues/303)
+        mod = self.module(asm_global_ctors)
+        ee = self.jit(mod)
+        ee.finalize_object()
+
+        ee.run_static_constructors()
+
+        # global variable should have been initialized
+        ptr_addr = ee.get_global_value_address("A")
+        ptr_t = ctypes.POINTER(ctypes.c_int32)
+        ptr = ctypes.cast(ptr_addr, ptr_t)
+        self.assertEqual(ptr.contents.value, 10)
+
+        foo_addr = ee.get_function_address("foo")
+        foo = ctypes.CFUNCTYPE(ctypes.c_int32)(foo_addr)
+        self.assertEqual(foo(), 12)
+
+        ee.run_static_destructors()
+
+        # destructor should have run
+        self.assertEqual(ptr.contents.value, 20)
+
+
 class TestGlobalVariables(BaseTest):
     def check_global_variable_linkage(self, linkage, has_undef=True):
         # This test default initializer on global variables with different
@@ -1149,26 +1419,88 @@ class TestInlineAsm(BaseTest):
     def test_inlineasm(self):
         llvm.initialize_native_asmparser()
         m = self.module(asm=asm_inlineasm)
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         asm = tm.emit_assembly(m)
         self.assertIn('nop', asm)
 
+
 class TestObjectFile(BaseTest):
+
+    mod_asm = """
+        ;ModuleID = <string>
+        target triple = "{triple}"
+
+        declare i32 @sum(i32 %.1, i32 %.2)
+
+        define i32 @sum_twice(i32 %.1, i32 %.2) {{
+            %.3 = call i32 @sum(i32 %.1, i32 %.2)
+            %.4 = call i32 @sum(i32 %.3, i32 %.3)
+            ret i32 %.4
+        }}
+    """
+
     def test_object_file(self):
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=False)
         mod = self.module()
         obj_bin = target_machine.emit_object(mod)
         obj = llvm.ObjectFileRef.from_data(obj_bin)
         # Check that we have a text section, and that she has a name and data
         has_text = False
+        last_address = -1
         for s in obj.sections():
             if s.is_text():
                 has_text = True
                 self.assertIsNotNone(s.name())
                 self.assertTrue(s.size() > 0)
                 self.assertTrue(len(s.data()) > 0)
+                self.assertIsNotNone(s.address())
+                self.assertTrue(last_address < s.address())
+                last_address = s.address()
                 break
         self.assertTrue(has_text)
 
+    def test_add_object_file(self):
+        target_machine = self.target_machine(jit=False)
+        mod = self.module()
+        obj_bin = target_machine.emit_object(mod)
+        obj = llvm.ObjectFileRef.from_data(obj_bin)
+
+        jit = llvm.create_mcjit_compiler(self.module(self.mod_asm),
+                                         target_machine)
+
+        jit.add_object_file(obj)
+
+        sum_twice = CFUNCTYPE(c_int, c_int, c_int)(
+            jit.get_function_address("sum_twice"))
+
+        self.assertEqual(sum_twice(2, 3), 10)
+
+    def test_add_object_file_from_filesystem(self):
+        target_machine = self.target_machine(jit=False)
+        mod = self.module()
+        obj_bin = target_machine.emit_object(mod)
+        temp_desc, temp_path = mkstemp()
+
+        try:
+            try:
+                f = os.fdopen(temp_desc, "wb")
+                f.write(obj_bin)
+                f.flush()
+            finally:
+                f.close()
+
+            jit = llvm.create_mcjit_compiler(self.module(self.mod_asm),
+                                             target_machine)
+
+            jit.add_object_file(temp_path)
+        finally:
+            os.unlink(temp_path)
+
+        sum_twice = CFUNCTYPE(c_int, c_int, c_int)(
+            jit.get_function_address("sum_twice"))
+
+        self.assertEqual(sum_twice(2, 3), 10)
+
+
 if __name__ == "__main__":
     unittest.main()
